rand('seed',pi);     % seed random number generator

% try to compile helper mex functions
mexfiles = {'nnsig.c','neteval.c'};
for ii=1:length(mexfiles)
  try
    eval( sprintf('mex %s',mexfiles{ii}) );
  catch
    fprintf('couldn''t compile %s:\n\t%s\nUsing m-file instead.\n', ...
      mexfiles{ii}, lasterr);
  end
end

% construct target function
nx = 9;     xx = linspace(-1, 2, nx)';
ny = 7;     yy = linspace(-1, 2, ny)';
[xm,ym] = meshgrid( xx, yy);
dm =  0.1 + 0.9*( xm >= 0 & xm <=1 & ym >= 0 & ym <= 1);

% network: use 5 hidden nodes and one output node
% bias introduced in input layer
a = 5;               % activation function parameter
nh = 9;   no = 1;   % network dimensions
W1 = 5*(rand(nh,3)-0.5);
W2 = 5*(rand(no,nh+1)-0.5);

% initial output, error surfaces
[om, em] = neteval(xx, yy, dm, W1, W2,a);
figure(3);     mesh(xx,yy, om);     title('Initial surface');
print -depsc backPropSig_3.eps
figure(4);     mesh(xx,yy,em);     title('Initial error surface');
print -depsc backPropSig_4.eps

% keep a record of the error function
icnt = 1;     Errv(icnt) = norm(em,'fro')^2;

icnt = icnt+1;      % icnt = "iteration count"
eta = 0.05;         % execute backprop algorithm with eta selected
mov = avifile('backPropSig.avi');
maxIter = 600;
for iter = 1:maxIter
  fprintf('%4d %s\n',iter,datestr(now));
  for ii=1:nx
    for jj = 1:ny
      y0 = [xx(ii); yy(jj)];            % input vector
      [y2,y1] = nnsig(y0, W1, W2, a);

      % perform backprop step
      e = dm(jj,ii) - y2;
      d2 = -e .* dphi(y2,a);
      DW2 = d2*[1;y1]';
      
      W2_times_d2 = W2' * d2;
      d1 = -(W2_times_d2(2:(nh+1))) .* dphi(y1,a) ;
      DW1 = d1*[1;y0]';

      W2 = W2 - eta*DW2;
      W1 = W1 - eta*DW1;

      % evaluate error surface
      [om,em] = neteval(xx,yy,dm, W1, W2,a);
      Errv(icnt) = norm(em,'fro')^2;
      icnt = icnt + 1;
    end
  end

  figure(6);     mesh(xx,yy, om);
  title(sprintf('Output surface iteration %d',icnt));
  axis([-1,2,-1,2,0,1.1]);
  text(-0.5,2,0.8,sprintf('current error =%12.4g',Errv(icnt-1)));
  A = getframe;      mov = addframe(mov,A);

  %print -djpeg -r0 myframe.jpg
  %A = imread('myframe.jpg','jpg');      mov = addframe(mov,A);

  if(iter == maxIter)
    print -depsc backPropSig_6.eps
    figure(7);      plot(Errv);     grid on; 
    title(sprintf('Error function per backprop step'))
    print -depsc backPropSig_7.eps
  end
end
mov = close(mov);
